
#######################################################################
##
##   Check for XFree extensions (DGA, VideoMode, DirectVideo)
##
#######################################################################

dnl AC_XFREE_EXTENSIONS
dnl Check for XFree extensions (DGA, VideoMode, DirectVideo)
dnl Defines HAVE_LIBXXF86VM, HAVE_LIBXXF86DGA, HAVE_LIBXDPMS
AC_DEFUN(AC_XFREE_EXTENSIONS,
[
xfreel=1
dgal=1
vmodel=1
dpmsl=1

AC_ARG_ENABLE(xfree,
    [  --disable-xfree         disable use of any XFree86 Extensions],
    if test "$enable_xfree" = "no"; then
	xfreel=0
    fi)

AC_ARG_ENABLE(dga,
    [  --disable-dga           disable use of XFree86 DirectVideo Extension],
    if test "$enable_dga" = "no"; then
	dgal=0
    fi)

AC_ARG_ENABLE(vmode,
    [  --disable-vmode         disable use of XFree86 VideoMode Extension],
    if test "$enable_vmode" = "no"; then
	vmodel=0
    fi)

AC_ARG_ENABLE(dpms,
    [  --disable-dpms          disable use of XFree86 DPMS Extension],
    if test "$enable_dpms" = "no"; then
	dpmsl=0
    fi)

AC_MSG_CHECKING([wheter to use XFree86 Extensions])
if test "$xfreel" = "1" ; then
	AC_MSG_RESULT([yes])
if test "$vmodel" = "1" ; then
AC_CACHE_VAL(vidmode_check,
[
    AC_MSG_CHECKING([for C++ save version of xf86vmode.h])
    AC_LANG_CPLUSPLUS
cat > conftest.$ac_ext <<EOF
#include <X11/Xlib.h>
#include <X11/Xproto.h>
#include <X11/Xutil.h>
#include <X11/extensions/xf86vmode.h>
#include <X11/extensions/xf86vmstr.h>
int vm_count;
int main() {
XF86VidModeModeInfo **vm_modelines;
return 0;
}
EOF
    ac_compile='${CXX-g++} -c $CXXFLAGS $all_includes $CPPFLAGS conftest.$ac_ext'
    if AC_TRY_EVAL(ac_compile); then
      AC_MSG_RESULT(yes)
      vidmode_check=1
      AC_CHECK_LIB(Xxf86vm, XF86VidModeQueryExtension,,, 
 	$X_LDFLAGS -lXext -lX11 $X_EXTRA_LIBS)
    else
      AC_MSG_RESULT(no)
      vidmode_check=0
    fi
])
fi

if test "$dgal" = "1" ; then
    AC_CHECK_LIB(Xxf86dga, XF86DGAQueryExtension,,, 
	    $X_LDFLAGS -lXext -lX11 $X_EXTRA_LIBS)
fi

if test "$dpmsl" = "1" ; then
    AC_CHECK_LIB(Xdpms, DPMSQueryExtension,,, 
	    $X_LDFLAGS -lXext -lX11 $X_EXTRA_LIBS)
fi

else
    AC_MSG_RESULT([no])
fi
])

#######################################################################
##
##   Check for Video4Linux and Video4Linux device
##
#######################################################################

dnl AC_VIDEO4LINUX
dnl Check for Video4Linux and Video4Linux device
dnl Defines V4L_DEVICE
AC_DEFUN(AC_VIDEO4LINUX,
[
withv4l2=0          # vl4 interface: v4l1: 0, v4l2: 1
v4ldir=""           # path to videodev.h 
kernel_v4l=0        # use kernel/external v4l driver
v4ldev=""           # /dev/video

AC_ARG_ENABLE(v4l2,
    [  --enable-v4l2           use Video4Linux Version 2],
	withv4l2=1 )

AC_MSG_CHECKING([for Video4Linux interface Version])

if test "$withv4l2" = "0" ; then

    AC_MSG_RESULT([1])

    AC_MSG_CHECKING(for videodev.h)

    if test "$kernel_v4l" = "1" ; then
	AC_EGREP_HEADER(video_mmap, linux/videodev.h,AC_DEFINE(USE_KERNEL_VIDEODEV) kernel_v4l=1,kernel_v4l=0)
	if test "$kernel_v4l" = "1" ; then
	    AC_MSG_RESULT("linux/videodev.h")
	    v4ldir=""
	else
	    if test -r ./videodev.h ; then
		    v4ldir="" 
		    AC_MSG_RESULT("./videodev.h")
	    else
		    AC_MSG_ERROR([Oops! No videodev.h found!])
	    fi
	fi
    else
	if test -r ./videodev.h ; then
		v4ldir="" 
		AC_MSG_RESULT("./videodev.h")
	else
		AC_MSG_ERROR([Oops! No videodev.h found!])
	fi
    fi
    AC_SUBST(v4ldir)

    AC_MSG_CHECKING(for Video4Linux device)
    for isdev in /dev/video /dev/video0 /dev/bttv
    do
	if test -w "$isdev"; then
		v4ldev="$isdev"
		break
	fi
    done
    if test -z $v4ldev ; then
	echo
	AC_MSG_ERROR([Oops! No Video4Linux device (/dev/video) found!])
    fi
    AC_DEFINE(V4L_DEVICE,"/dev/video")
    AC_MSG_RESULT($v4ldev)

else # v4l2

    AC_MSG_RESULT([2])
    AC_MSG_ERROR(Video4Linux2 not implemented yet!)

fi
])

#######################################################################
##
##   Check for use of VideoText/TeleText
##
#######################################################################

dnl AC_VTX
dnl Check for use of VideoText/TeleText
dnl Defines HAVE_VTX, HAVE_DEVVTX, HAVE_DEVVBI 
AC_DEFUN(AC_VTX,
[
hvtx=0
AC_ARG_ENABLE(vtx,
    [  --enable-vtx            enable VideoText/TeleText],
    if test "$enable_vtx" != "no"; then
	hvtx=1
    else
	hvtx=0
    fi)

if test "$hvtx" = "1" ; then
    echo
    echo "----------------------------------------------------------------------"
    echo " WARNING: The videotext package is still very experimental!"
    echo " So it might not compile or work as you expect it!"
    echo "----------------------------------------------------------------------"
    echo

    AC_MSG_CHECKING([for vtx device])
    if test -w "/dev/vtx"; then
	AC_MSG_RESULT([/dev/vtx])
	AC_DEFINE(HAVE_DEVVTX)	
    else
	AC_MSG_RESULT([not found])
    fi

    AC_MSG_CHECKING([for vbi device])
    if test -w "/dev/vbi"; then
	AC_MSG_RESULT([/dev/vbi])
	AC_DEFINE(HAVE_DEVVBI)
    else
	AC_MSG_RESULT([not found])
    fi

    #OUTPUT_MAKEFILES=$OUTPUT_MAKEFILES" libvtx/Makefile"
    #MAKE_SUBDIRS=$MAKE_SUBDIRS" libvtx"

    #LIBVTX_LIB="-lvtx"
    #AC_SUBST(LIBVTX_LIB)
    #LIBVTX_DIRI="-I../libvtx"
    #AC_SUBST(LIBVTX_DIRI)
    #LIBVTX_DIRL="-L$topdir/libvtx"
    #AC_SUBST(LIBVTX_DIRL)

    AC_DEFINE(HAVE_VTX)

fi
])

#######################################################################
##
##   Check for LIRC
##
#######################################################################

## lirc.m4 (Macros for autoconf)
##
## (C) 1999 Christoph Bartelmus (lirc@bartelmus.de)
## 

dnl AC_PATH_LIRC([MINIMUM-VERSION])
dnl Check for LIRC and define LIRCD
dnl
AC_DEFUN(AC_PATH_LIRC,
[
  min_lirc_version=ifelse([$1], ,0.5.5,$1)
  AC_MSG_CHECKING(for LIRC>=$min_lirc_version)

  AC_CACHE_VAL(ac_cv_have_lirc,[
    no_lirc=no
    lirc_version=none
    lirc_cross_compiling=no

    if test ! -S /dev/lircd; then
      LIRCD=/dev/null
      no_lirc=yes
    else
      LIRCD=/dev/lircd

      rm -f conf.lirc
      ac_save_cflags="${CFLAGS}"
      CFLAGS="$CFLAGS -DLIRCD=\"$LIRCD\" -I/usr/local/include"
      AC_TRY_RUN(
[
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <errno.h>
#include <signal.h>
#include <limits.h>

#define PACKET_SIZE 256
/* three seconds */
#define TIMEOUT 3

int timeout=0;

void sigalrm(int sig)
{
	timeout=1;
}

const char *read_string(int fd)
{
	static char buffer[PACKET_SIZE+1]="";
	char *end;
	static int ptr=0;
	ssize_t ret;
		
	if(ptr>0)
	{
		memmove(buffer,buffer+ptr,strlen(buffer+ptr)+1);
		ptr=strlen(buffer);
		end=strchr(buffer,'\n');
	}
	else
	{
		end=NULL;
	}
	alarm(TIMEOUT);
	while(end==NULL)
	{
		if(PACKET_SIZE<=ptr)
		{
			ptr=0;
			return(NULL);
		}
		ret=read(fd,buffer+ptr,PACKET_SIZE-ptr);

		if(ret<=0 || timeout)
		{
			if(!timeout)
			{
				alarm(0);
			}
			ptr=0;
			return(NULL);
		}
		buffer[ptr+ret]=0;
		ptr=strlen(buffer);
		end=strchr(buffer,'\n');
	}
	alarm(0);timeout=0;

	end[0]=0;
	ptr=strlen(buffer)+1;
	return(buffer);
}

enum packet_state
{
	P_BEGIN,
	P_MESSAGE,
	P_STATUS,
	P_DATA,
	P_N,
	P_DATA_N,
	P_END
};

char *get_version(int fd,const char *packet)
{
	int done,todo;
	const char *string,*data;
	char *endptr;
	enum packet_state state;
	int status,n;
	unsigned long data_n;
	unsigned int major,minor,micro;
	static char version[100];

	todo=strlen(packet);
	data=packet;
	while(todo>0)
	{
		done=write(fd,(void *) data,todo);
		if(done<0)
		{
			return(NULL);
		}
		data+=done;
		todo-=done;
	}

	/* get response */
	status=0;
	state=P_BEGIN;
	n=0;
	while(1)
	{
		string=read_string(fd);
		if(string==NULL) return(NULL);
		switch(state)
		{
		case P_BEGIN:
			if(strcasecmp(string,"BEGIN")!=0)
			{
				continue;
			}
			state=P_MESSAGE;
			break;
		case P_MESSAGE:
			if(strncasecmp(string,packet,strlen(string))!=0 ||
			   strlen(string)+1!=strlen(packet))
			{
				state=P_BEGIN;
				continue;
			}
			state=P_STATUS;
			break;
		case P_STATUS:
			if(strcasecmp(string,"SUCCESS")==0)
			{
				status=0;
			}
			else if(strcasecmp(string,"END")==0)
			{
				status=0;
				return(NULL);
			}
			else if(strcasecmp(string,"ERROR")==0)
			{
				status=-1;
			}
			else
			{
				goto bad_packet;
			}
			state=P_DATA;
			break;
		case P_DATA:
			if(strcasecmp(string,"END")==0)
			{
				return(NULL);
			}
			else if(strcasecmp(string,"DATA")==0)
			{
				state=P_N;
				break;
			}
			goto bad_packet;
		case P_N:
			errno=0;
			data_n=strtoul(string,&endptr,0);
			if(!*string || *endptr)
			{
				goto bad_packet;
			}
			if(data_n==0)
			{
				state=P_END;
			}
			else
			{
				state=P_DATA_N;
			}
			break;
		case P_DATA_N:
			if(data_n==1 && status==0)
			{
				if(sscanf(string,"%u.%u.%u",
					  &major,&minor,&micro)==3)
				{
					sprintf(version,"%u.%u.%u",
						major,minor,micro);
				}
				else
				{
					goto bad_packet;
				}
			}
			n++;
			if(n==data_n) state=P_END;
			break;
		case P_END:
			if(strcasecmp(string,"END")==0)
			{
				return(version);
			}
			goto bad_packet;
			break;
		}
	}
 bad_packet:
	return(NULL);
}

int main(int argc,char **argv)
{
	char *version,*min_version;
	unsigned int major,minor,micro,min_major,min_minor,min_micro;
	struct sockaddr_un addr;
	FILE *result;
	int fd;
	struct sigaction act;

	result=fopen("conf.lirc","w");
	if(result==NULL) exit(EXIT_FAILURE);
	
	act.sa_handler=sigalrm;
	sigemptyset(&act.sa_mask);
	act.sa_flags=0;           /* we need EINTR */
	sigaction(SIGALRM,&act,NULL);

	addr.sun_family=AF_UNIX;
	strcpy(addr.sun_path,LIRCD);
	fd=socket(AF_UNIX,SOCK_STREAM,0);
	if(fd==-1)
	{
		fprintf(result,"unknown");
		fclose(result);
		close(fd);
		exit(EXIT_FAILURE);
	};
	if(connect(fd,(struct sockaddr *)&addr,sizeof(addr))==-1)
	{
		fprintf(result,"unknown");
		fclose(result);
		close(fd);
		exit(EXIT_FAILURE);
	};

	version=get_version(fd,"VERSION\n");
	if(version==NULL)
	{
		fprintf(result,"<0.5.5");
		fclose(result);
		close(fd);
		exit(EXIT_FAILURE);
	}
	fprintf(result,"%s",version);
	fclose(result);
	close(fd);

	/* HP/UX 9 (%@#!) writes to sscanf strings */

	min_version=strdup("$min_lirc_version");
	if(min_version==NULL) exit(EXIT_FAILURE);

	if(sscanf(version,"%u.%u.%u",&major,&minor,&micro)!=3 ||
	   sscanf(min_version,"%u.%u.%u",&min_major,&min_minor,&min_micro)!=3)
	{
		exit(EXIT_FAILURE);
	}
	if(major<min_major ||
	   (major==min_major && minor<min_minor) ||
	   (major==min_major && minor==min_minor && micro<min_micro))
	{
		exit(EXIT_FAILURE);
	}

	exit(EXIT_SUCCESS);
}
],
        if test -f conf.lirc; then
          lirc_version=`cat conf.lirc`

##        lirc_major_version=`cat lirc.conf | \
##          sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
##        lirc_minor_version=`cat lirc.conf | \
##          sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
##        lirc_micro_version=`cat lirc.conf | \
##          sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
        else
          no_lirc=yes
        fi,
        no_lirc=yes
        if test -f conf.lirc; then
          lirc_version=`cat conf.lirc`
        else
          lirc_version=none
        fi
        ,lirc_cross_compiling=yes
      )
## AC_TRY_RUN()      
      rm -f conf.lirc
      CFLAGS="$ac_save_CFLAGS"
    fi
    ac_cv_have_lirc="no_lirc=${no_lirc} lirc_version=\"${lirc_version}\" \
                     lirc_cross_compiling=${lirc_cross_compiling} \
                     LIRCD=${LIRCD}"
  ])
## AC_CACHE_VAL

  eval "$ac_cv_have_lirc"

  if test x${no_lirc} = xyes; then
    if test x${lirc_version} = xunknown; then
      AC_MSG_RESULT([missing (lircd not running ?)])
    elif test x${lirc_version} != xnone; then
      AC_MSG_RESULT([missing (found lirc-${lirc_version})])
    else
      AC_MSG_RESULT(missing)
    fi
  else
    if test x${lirc_cross_compiling} = xyes; then
      AC_MSG_RESULT(found)
    else
      AC_MSG_RESULT([found lirc-${lirc_version}])
    fi
  fi

  AC_DEFINE_UNQUOTED(LIRCD,"${LIRCD}")
]
)
