//-> Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)
/* 
    xWinTV using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
//<-

//-> definitions
//#define withTVscreen
//<-

//-> includes
#include <stdio.h>
#include <stdlib.h>

#include "wintv.h"

#include "xrwevent.h"
#include "wintvmain.h"  // immer in dieser Reihenfolge...
#if defined  HAVE_VTX || HAVE_VBI 
#include <winvtxmain.h>
#endif
#include "wintvchannel.h"
#include "wintvstream.h"

#include <kapp.h>
#include <parsecl.h>

#if QT_VERSION >= 200 // communication with windows manager
#include <klocale.h>
#include <khelpmenu.h> 
#include <kglobal.h>
#include <kstddirs.h>
#include <kcmdlineargs.h> 
#include <kaboutdata.h>
#endif

// X11 stuff
#include <X11/Xlib.h>
#ifdef X11DEBUG
#include "xeventtype.h"
#endif
//<-


//-> class TVApplication: public KApplication
class TVApplication: public KApplication
{
  Q_OBJECT

public:
  TVApplication( int & argc, char ** argv );
  ~TVApplication();

  bool x11EventFilter( XEvent * );
  void setFilterObject(QObject *theObject) { winTVscreenObject = theObject; } 

public slots:
  void showMixerSLOT(int id);

protected slots:
  void shutDownSLOT();
  void saveYourselfSLOT();
  void sessionManagerSLOT();

protected:
  bool queryExit();
  
private:
  bool shutdownDone;
  void destroyTV();

  QObject *winTVscreenObject;

  rcfile *rc;
  parsecl *pcl;

  winTVChannelWidget *winTVChannel;
  winTVVClipWidget *winTVVClip;
  winTVMainWidget *winTVMain;
  winMixer *winTVMixer[MAX_MIXER_DEV];
#ifdef HAVE_VTX
  winVTXMainWidget *winVTXMain;
#endif
};
//<-

TVApplication::TVApplication( int & argc, char ** argv ) : KApplication(argc, argv) 
{
  rc = NULL;

#ifdef myDEBUG
  debug("parse command line ...");
#endif

  pcl = NULL;

#if QT_VERSION < 200
  pcl= new parsecl( argc, argv );
  pcl->addOption(OP_BPP,"-b","--bpp","n","color depth of the display is n (n=8,15,16,24,32)");
  pcl->addOption(OP_PAL,"-p","--palette","n","set palette of display (see help for details)");
  pcl->addOption(OP_SHI,"-s","--shift","<value>","shift frame buffer (image) by <value> bytes");
  //pcl->addOption(OP_DOK,"-n","--nodock",NULL,"disable docking of TV windwo");
  pcl->addOption(OP_REF,"-r","--refresh",NULL,"do smart refreshs");
  pcl->addOption(OP_DEV,"-l","--device","<file>","use <file> as video4linux device");
  pcl->addOption(OP_CAP,"-c","--capture","[on|off]","force capture mode to be on or off");
  pcl->addOption(OP_VME,"-x","--no_vme",NULL,"disable XFree extentions (VidMode Extension)");
  
  pcl->parse();
#endif

#ifdef myDEBUG
  debug("... loading rc ...");
#endif

  /////////////////////////////////////////////////////////////////////////////
  // load rc-file
  rc=new rcfile();
  rc->load();
  rc->checkVersion();

#ifdef myDEBUG
  debug("TVApplication: init widgets");
#endif
  int i;
  for ( i = 0; i < mixer::getNumMixer(); i++ )
    winTVMixer[i]= new winMixer(rc);
  winTVMain= new winTVMainWidget( pcl, rc );
  winTVChannel= new winTVChannelWidget( rc );
  winTVVClip= new winTVVClipWidget( rc );

  setFilterObject(winTVMain->winTVScreenPtr());
  setMainWidget(winTVMain);

  // connect TVApplication
  connect(this,SIGNAL(shutDown()),this,SLOT(shutDownSLOT()));
  connect(this,SIGNAL(saveYourself()),this,SLOT(saveYourselfSLOT()));
 
  // connect winTVMain and winTVChannel
  connect(winTVMain,SIGNAL(channelChangedSIGNAL()),
		   winTVChannel,SLOT(activeChannelChangedSLOT()));
  connect(winTVMain,SIGNAL(newChannelListSIGNAL(QList<channel> *)),
		   winTVChannel,SLOT(newChannelListSLOT(QList<channel> *)));
  connect(winTVMain,SIGNAL(toggleChannelBoxSIGNAL( QPoint * )),
		   winTVChannel,SLOT(toggleVisibilitySLOT( QPoint * )));

  // connect winTVMixer and winTVMain
  connect(winTVMixer[rc->mix_master],SIGNAL(volChanged(int, int)),
	  winTVMain,SLOT(volumeSLOT(int, int)));
  connect(winTVMixer[rc->mix_master],SIGNAL(volMuteChanged(bool)),
	  winTVMain,SLOT(volMuteChangedExtSLOT(bool)));
  // connect winTVMain and winTVMixer
  connect(winTVMain,SIGNAL(volumeChangedSIGNAL(int,int)),
	  winTVMixer[rc->mix_master],SLOT(extVolSet(int,int)));
  connect(winTVMain,SIGNAL(volMuteChangedSIGNAL(bool)),
	  winTVMixer[rc->mix_master],SLOT(extVolMute(bool)));
  connect(winTVMain,SIGNAL(toggleMixerBoxSIGNAL(int)),
  	  this,SLOT(showMixerSLOT(int)));

  // connect winTVScreen and winTVMixer
  connect(winTVMain->winTVScreenPtr(),SIGNAL(requestMixerMute(bool)),
	  winTVMixer[rc->mix_master],SLOT(extMuteRequest( bool )));

  // edit, cut, copy and paste channel
  connect(winTVChannel,SIGNAL(channelSelectedSIGNAL()),
	  winTVMain,SLOT(channelChangedSLOT()));
  connect(winTVChannel,SIGNAL(addChannelSIGNAL()),
	  winTVMain,SLOT(addChannelSLOT()));
  connect(winTVChannel,SIGNAL(editChannelSIGNAL()),
	  winTVMain,SLOT(editChannelSLOT()));
  connect(winTVChannel,SIGNAL(deleteChannelSIGNAL()),
	  winTVMain,SLOT(deleteChannelSLOT()));
  connect(winTVChannel,SIGNAL(copyChannelSIGNAL()),
	  winTVMain,SLOT(copyChannelSLOT()));
  connect(winTVChannel,SIGNAL(pasteChannelSIGNAL()),
	  winTVMain,SLOT(pasteChannelSLOT()));

  // connect winTVMain and winTVVClip
  connect(winTVMain,SIGNAL(toggleVClipSIGNAL()),
		   winTVVClip,SLOT(toggleVClipSLOT()));

  winTVChannel->setPopup(winTVMain->getChannelMenu());

  winTVMain->show();

  KConfig *config=GETAPPCONFIGINST;
  for ( i = 0; i < mixer::getNumMixer(); i++ ) {
    config->setGroup("CONFIG__mixer-"+QString().setNum(i));
    //winTVMixer[rc->mix_master]->setV4lifP(winTVMain->winTVScreenPtr()->v4lxifP());
    if ( config->readNumEntry("was_visible",0) && winTVMixer[i]->mixerEnabled() ) 
      winTVMixer[i]->show();
  }

  if ( rc->ol_ch_wasvis ) winTVChannel->show();

  winTVMain->emitSetupSignals();

#ifdef HAVE_VTX
  winVTXMain= new winVTXMainWidget( pcl );
  winVTXMain->show();
  winVTXMain->postInit();
#endif

  shutdownDone = false;
}

void TVApplication::showMixerSLOT(int id)
{
  winTVMixer[id]->showMixer();
}

void TVApplication::destroyTV()
{
  if ( shutdownDone ) return;
  shutdownDone = true;

  setFilterObject(NULL);
  
  delete winTVVClip;
  rc->ol_ch_wasvis= winTVChannel->isVisible();
  delete winTVChannel;
  delete winTVMain;

  KConfig *config=GETAPPCONFIGINST;
  for ( int i = 0; i < mixer::getNumMixer(); i++ ) {
    config->setGroup("CONFIG__mixer-"+QString().setNum(i));
    config->writeEntry("was_visible",(int)winTVMixer[i]->isVisible());
    delete winTVMixer[i];
  }

#ifdef HAVE_VTX
  delete winVTXMain;
#endif

  delete rc;
  if ( pcl != NULL ) delete pcl;

  rc = NULL;

#ifdef myDEBUG
  debug("TVApplication::destroyTV end");
#endif
}

TVApplication::~TVApplication()
{
  destroyTV();
}

//-> bool TVApplication::x11EventFilter( XEvent *event)
bool TVApplication::x11EventFilter( XEvent *event )
{
  Window checkw1,checkw2,wwin;

  wwin=DefaultRootWindow(qt_xdisplay());
  checkw1=event->xmap.event;
  checkw2=event->xany.window; 

  if (checkw1 == wwin) {

    X11RWEvent *xrwe = new X11RWEvent(XRWEvent,
				      event->type,
				      event);
    postEvent(winTVscreenObject,xrwe);
    
#ifdef X11DEBUG
    debug("x11EF cwin1 xmap type: %d = %s",event->type,events[event->type]);
#endif

  return TRUE;
  } else if (checkw2 == wwin) {
#ifdef X11DEBUG
  debug("x11EF cwin2 xmap type: %d = %s",event->type,events[event->type]);
#endif
  return TRUE; 
  }

 return FALSE;
}
//<-

void TVApplication::shutDownSLOT()
{
#ifdef myDEBUG
  debug("TVApplication::shutDownSLOT()");
#endif
  destroyTV();
#ifdef myDEBUG
  debug("TVApplication::shutDownSLOT() end");
#endif
}

void TVApplication::saveYourselfSLOT()
{
#ifdef myDEBUG
  debug("TVApplication::saveYourselfSLOT()");
#endif

  if ( rc != NULL ) 
    rc->save();
}

bool TVApplication::queryExit()
{
#ifdef myDEBUG
  debug("TVApplication::queryExit()");
#endif
  destroyTV();
#ifdef myDEBUG
  debug("TVApplication::queryExit() end");
#endif
  return true;
}

void TVApplication::sessionManagerSLOT()
{
#ifdef myDEBUG
  debug("sessionManagerSLOT, trying to save rc...");
#endif
}

//-> void myMessageOutput( QtMsgType type, const char *msg )
void myMessageOutput( QtMsgType type, const char *msg )
{
  switch ( type ) {
  case QtDebugMsg:
    fprintf( stderr, "Debug: %s\n", msg );
    break;
  case QtWarningMsg:
    fprintf( stderr, "Warning: %s\n", msg );
    break;
  case QtFatalMsg:
    fprintf( stderr, "Fatal: %s\n", msg );
    //abort();                        // dump core on purpose
    exit(1);
  }
}
//<-

//-> int main( int argc, char **argv )
//
int main( int argc, char **argv )
{
  qInstallMsgHandler( myMessageOutput );

#ifdef myDEBUG
  debug("we begin ...");
#endif

#if QT_VERSION >= 200
#ifdef myDEBUG
  debug("call KCmdLineArgs::init");
#endif
  KAboutData aboutData("kwintv","kWinTV",
		       VERSION,
		       "kWinTV is a KDE application for TV cards\n"
		       "supported by a Video4Linux compatible driver.",
		       KAboutData::License_GPL, 
		       "Copyright 1998, 1999, 2000 by Moritz Wenk",
		       0,
		       "http://kwintv.sourceforge.net",
		       "wenk@swyx.com");

  aboutData.addAuthor("Moritz Wenk",0,"wenk@swyx.com","http://kwintv.sourceforge.net");

  KCmdLineArgs::init( argc, argv, &aboutData );

  static KCmdLineOptions options[] = {
	{ "bpp <pits per pixel>", I18N_NOOP("color depth of the display is <pits per pixel>"), 0 },
	{ "b", 0, 0 },
	{ "palette <id>", I18N_NOOP("force used palette for display to <id>"), 0 }, 
	{ "p", 0, 0 },
	{ "shift <int>", I18N_NOOP("shift frame buffer (TV image) by <int> bytes"), 0},
	{ "s", 0, 0 },
	{ "refresh <mode>", I18N_NOOP("set refresh mode to <mode>"), 0 },
	{ "r", 0, 0 },
	{ "device <dev>", I18N_NOOP("use device <dev> as video4linux device"), "/dev/video" },
	{ "d", 0, 0 },
	{ "noxfree", I18N_NOOP("disable XFree extentions (VidMode, DGA, DV)"), 0 },
	{ "x", 0 ,0 }
  };
  KCmdLineArgs::addCmdLineOptions(options);
#endif

#ifdef myDEBUG
  debug("call TVApplication");
#endif

  KApplication::setColorSpec( KApplication::ManyColor ); 

  TVApplication *winTVApplication= new TVApplication( argc, argv );

#if QT_VERSION < 200
  winTVApplication->enableSessionManagement( true );
  winTVApplication->setWmCommand(SLOT(sessionManagerSLOT()));
#endif

#ifdef myDEBUG
  debug("... runing ...");
#endif

  // main loop
  winTVApplication->exec();

  // the end
  delete winTVApplication;

#ifdef myDEBUG
  debug("... end");
#endif

  return 0;
}
//<-

//-> moc's
#include "wintv.moc"
//<-








