/* 
    xWinTV using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef lswidget_included
#define lswidget_included

// uncomment to enable drag & drop
//#define DRAGDROP

#include <wintv.h>

#include <qapp.h>
#include <qlist.h>
#include <qpixmap.h>
#include <qwidget.h>
#include <qlistview.h> 
#include <qtooltip.h>
#include <qpopmenu.h>
#include <qbutton.h>
#include <qpushbutton.h> 
#include <qintdict.h> 
#include <qlineedit.h> 
#include <qslider.h> 

#include <kapp.h>
#include <ktoolbar.h>
#include <ktmainwindow.h>
#include <kconfig.h>

#include <rcfile.h>
#include <channel.h>
#include <mixer.h>
#include <v4lxif.h>

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

class winSlider : public QWidget
{
  Q_OBJECT

public:
  winSlider ( int devID, int _min=0, int _max=100, 
	      int _valr=0, int _vall=0,
	      bool _stereo= false, bool _input= false, 
	      bool _recsrc= false, bool _muteable= false, 
	      QPixmap * pix= NULL, QWidget * parent=0, const char * name=0 );
  ~winSlider ();

  bool isSplited() { return splited; }
  bool isMuted() { return muted; }
  bool isEnabled() { return mm; }
  bool isRecSrc() { return recsrc; }

  int lVol() { return lval; }
  int rVol() { return rval; }

public slots:
  void setLValue( int );
  void setRValue( int );
  void split( bool );  // true: both, false: one slider
  void mute( bool );
  void crecsrc( bool );
  void setEnabled( bool ); // true: enabled, false: disabled

signals:
  void valueChanged( int, int, int );
  void splitChanged( int, bool );
  void muteChanged( int, bool );
  void recsrcChanged( int, bool );

private slots:
  void valueLChangedI( int );
  void valueRChangedI( int );
  void popupSLOT( int id );

protected:
  void resizeEvent ( QResizeEvent * );
  bool eventFilter( QObject *o, QEvent *e );

private:
  QSlider *sleft,*sright;
  QPopupMenu *sPopup;
  QLabel * icon;
  int devID;
  int sid[10];
  int min,max;
  int lval,rval;
  int yoff;
  bool stereo,input,muteable;
  bool splited,muted; 
  bool mm,recsrc;
};

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

class winMixer : public QWidget
{
  Q_OBJECT

public:
  winMixer( rcfile * _rc= 0, QWidget * parent=0, const char * name=0 );
  ~winMixer();

  //void setV4lifP( v4lxif * _if ) { v4l=_if; }
  bool mixerEnabled() { return haveMixer; }

public slots:
  void extVolSet( int, int );
  void extVolMute( bool );

  void showMixer();
  
  void extMuteRequest(bool);

signals:
  void volChanged( int, int );
  void volLChanged( int );
  void volRChanged( int );
  void volMuteChanged( bool );

private slots:
  void changeVol( int, int, int );
  void changeSize( int, bool );
  void changeMute( int, bool );
  void changeRecsrc( int, bool );
  //void volMuteI( int, bool );
  
protected:
  void resizeEvent ( QResizeEvent * );

private:
  rcfile * rc;
  QIntDict<winSlider> *slidlist;
  mixer * kmix;
  //v4lxif * v4l;
  KConfig *config;
  bool haveMixer;
  int xoff;
  static int glbId;
  int myId;
};

//  class winMixerWindow : public KTopLevelWidget
//  {
//  public:
//    winMixerWindow( rcfile * _rc= 0, QWidget * parent=0, const char * name=0 );
//    ~winMixerWindow();

//  private:
//    rcfile * rc;

//  }


/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

class winProgramItem;

class winProgramList : public QListView
{
  Q_OBJECT

public:
  winProgramList( QWidget * parent=0, const char * name=0 );
  ~winProgramList();

  void setCPopup(QPopupMenu *sp) { cPopup=sp; }
  void setKeyTimeOut(int time) { keyTimeOut=time; }

  bool setCurrentProgramItem( int item );

  void hideLineEdit();

protected:
  void mouseReleaseEvent ( QMouseEvent * me);
#ifdef DRAGDROP
  void mousePressEvent ( QMouseEvent * me);
  void mouseMoveEvent ( QMouseEvent * me);
#endif
  void keyPressEvent ( QKeyEvent * );
  void leaveEvent ( QEvent * );

signals:
  void moveItemSIGNAL( QListViewItem *, QListViewItem * );
  void channelTextChangedSIGNAL(QListViewItem * item);

protected slots:
  void keyTimerSLOT();
  //#if QT_VERSION < 200
  void textChangedSLOT( const char * );
  //#else
  void textChangedSLOT( const QString& );
  //#endif
  void returnPressedSLOT(); 
  void doubleClickedSLOT( QListViewItem *item );
  void selectionChangedSLOT( QListViewItem * );
  //void resizeEvent( QResizeEvent * );

private:
  QPopupMenu *cPopup;
  winProgramItem *toMoveItem, *lastVisItem;
  winProgramItem *eitem;

  int pressedButton;
  int oldY;
  bool itemWasMoved;
  bool cursorChanged;

  QTimer *keyTimer;
  int key1;
  int key2;
  int keyTimeOut;
  
  QLineEdit *lineedit;
  bool returnpressed;
};

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

class winTVChannelWidget : public KTMainWindow
{
  Q_OBJECT

public:
  enum programNumberStyleEnum { 
    DefaultStyle = 0, 
    EuropeanStyle, 
    AmericanStyle, 
    MaxNumStyle };

  winTVChannelWidget(rcfile *_rc=NULL, QWidget* parent = NULL,const char* name = NULL );
  ~winTVChannelWidget();

  void setPopup(QPopupMenu *cp);

public slots:
  void newChannelListSLOT(QList<channel> *clist);
  void activeChannelChangedSLOT();
  void toggleVisibilitySLOT( QPoint *to );

protected slots:
  void buttonsSLOT(int);
  void programSelectedSLOT(QListViewItem *);
  void channelEditEmitSLOT();
  void moveItemSLOT(int from, int to);

#ifdef SHELLKEYS
  //void keyTimerSLOT();

protected:
  void keyPressEvent ( QKeyEvent * );
#endif

signals:
  void channelSelectedSIGNAL();
  void addChannelSIGNAL();
  void editChannelSIGNAL();
  void deleteChannelSIGNAL();
  void copyChannelSIGNAL();
  void pasteChannelSIGNAL();

private:
  winProgramList * programlist;
  KToolBar *toolbar;

  QPixmap  pm;
  QPixmap  ch_add_xpm, ch_edit_xpm, ch_delete_xpm, mup_xpm, mdown_xpm, cs_xpm;
  
  rcfile *rc;
  KConfig *config;

  QList<channel> *clistp;

  bool inserting;

  QTimer *keyTimer;
  int key1;
  int key2;

  programNumberStyleEnum programNumberStyle;
};


/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

#define VIEW_COUNT 4

class winProgramItem : public QListViewItem
{
public:
  enum ViewAtoms { ViewNumber   = 0x01,
		   ViewChannel  = 0x02,
		   ViewName     = 0x04,
		   ViewFreq     = 0x08,
                   ViewAll      = 0xff   };

  winProgramItem( winTVChannelWidget::programNumberStyleEnum style,
		  channel * chan, int _view=ViewAll, QListView * parent=0, const char * name=0 );

  channel * getChannelP() { return pchannel; }
  virtual void setCText ( int, const char * ); 

private:
  channel * pchannel;
  int view;
};

#endif


// local Variables: ***
// compile-command: "make v4lx" ***
// End: ***









