/* 
    kwintv using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "wintvscan.h"
#include <kapp.h>
#include <klocale.h>

#ifdef myDEBUG
#define wsDEBUG
#endif

winTVScanWizard::winTVScanWizard(QWidget *parent, const char *name)
  : KWizard(parent,name)
{
  setCaption((const char*)(QString("kWinTV ")+VERSION+i18n(" - ScanWizard")));

#if 0
  pm = loader->loadIcon("kwintv_48x48.png");
  setIcon(pm);
  setIconText("kWinTV");
#endif

  wizstate = WizCanceled;
  scanwasdone=false;

  wizstart = new wizFinish();
  wiznorm = new wizNorm();
  wizregion = new wizRegion();
  wizscan = new wizScan();
  wizfinish = new wizFinish();

  wizfinish->ShowFinishText();
  wizstart->ShowStartText();

#if QT_VERSION < 200
  wizPstart.title = "Start ScanWizard";
  wizPstart.w = (QWidget*)wizstart;
  wizPstart.enabled = true;
  wizPstart.id = 0;

  wizPnorm.title = "Select your input norm";
  wizPnorm.w = (QWidget*)wiznorm;
  wizPnorm.enabled = true;
  wizPnorm.id = 1;

  wizPregion.title = "Select your receiving region";
  wizPregion.w = (QWidget*)wizregion;
  wizPregion.enabled = true;
  wizPregion.id = 2;

  wizPscan.title = "Scan for Programs";
  wizPscan.w = (QWidget*)wizscan;
  wizPscan.enabled = true;
  wizPscan.id = 3;

  wizPfinish.title = "Finish ScanWizard";
  wizPfinish.w = (QWidget*)wizfinish;
  wizPfinish.enabled = true;
  wizPfinish.id = 4;

  addPage(&wizPstart);
  addPage(&wizPnorm);
  addPage(&wizPregion);
  addPage(&wizPscan);
  addPage(&wizPfinish);

  gotoPage(wizPstart.id);

  setCancelButton();
  setOkButton(i18n("Finish"));

  okbutton = getOkButton();
  okbutton->setEnabled(false);

  connect(this,SIGNAL(selected(int)),this,SLOT(selectedSLOT(int)));
  connect(this,SIGNAL(okclicked()),this,SLOT(okClickedSLOT()));
  connect(this,SIGNAL(cancelclicked()),this,SLOT(cancelClickedSLOT()));

#else // QT 2.x

  addPage(wizstart,"Start ScanWizard");
  addPage(wiznorm,"Select your input norm");
  addPage(wizregion,"Select your receiving region");
  addPage(wizscan,"Scan for Programs");
  addPage(wizfinish,"Finish ScanWizard");

  setBackEnabled(wizstart,false);
  setNextEnabled(wizfinish,false);
  setFinishEnabled(wizfinish,true);

  connect(this,SIGNAL(selected(const QString &)),this,SLOT(selectedSLOT(const QString &)));

#endif

  connect(wizscan,SIGNAL(wizScanScanReadySIGNAL()),this,SLOT(scanFinishedSLOT()));

  resize(520,340);
  
}

winTVScanWizard::~winTVScanWizard()
{
  delete wizstart;
  delete wiznorm;
  delete wizregion;
  delete wizscan;
  delete wizfinish;

  channellist.clear();
}

void winTVScanWizard::reset()
{
#if QT_VERSION < 200
  gotoPage(wizPstart.id);
#else
  showPage(wizstart);
#endif
}

void winTVScanWizard::selectedSLOT(const QString & pagename)
{
#ifdef wsDEBUG
  debug("wintvscan: page %s",(const char*)pagename);
#endif

  if ( currentPage() != lastpage && (wizScan*)lastpage == wizscan ) 
    wizscan->wizPBStopSLOT();

  if ( (wizScan*)currentPage() == wizscan ) {
    wizscan->setScanInfo(&channellist,
			 wiznorm->getNorm(),
			 wizregion->getRegion(),
			 wizregion->getScanAll());
  }

  lastpage = currentPage();
}

void winTVScanWizard::selectedSLOT(int id)
{
#if QT_VERSION < 200
  if ( lastpage == wizPscan.id && id != wizPscan.id ) 
    wizscan->wizPBStopSLOT();

  if ( id == wizPscan.id ) {
    wizscan->setScanInfo(&channellist,
			 wiznorm->getNorm(),
			 wizregion->getRegion(),
			 wizregion->getScanAll());
  } else if ( id == wizPfinish.id ) {
    okbutton->setEnabled(scanwasdone);
  } 
    
  lastpage = id;
#endif
}

void winTVScanWizard::okClickedSLOT()
{
  wizscan->wizPBStopSLOT();
  wizstate = WizOk;
  emit endWizardSIGNAL();
}

void winTVScanWizard::cancelClickedSLOT()
{
  wizscan->wizPBStopSLOT();
  channellist.clear();
  wizstate = WizCanceled;
  emit endWizardSIGNAL();
}

void winTVScanWizard::scanFinishedSLOT()
{
#ifdef wsDEBUG
  debug("wintvscan: scanFinishedSLOT");
#endif
  scanwasdone=true;
#if QT_VERSION < 200
  okbutton->setEnabled(scanwasdone);
#endif
}

#include "wintvscan.moc"










