/* 
    xWinTV using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <wintv.h>
#include <qfiledlg.h> 
#include <qmsgbox.h>

#include <kapp.h>
#include <kiconloader.h>
#include <kmenubar.h>
#include <klocale.h>

#include <wintvstream.h>

#ifdef myDEBUG
#define vcDEBUG
#endif

#define SB_FRAMES_TXT 1
#define SB_FRAMES     2
#define SB_FLOST_TXT  3
#define SB_FLOST      4
#define SB_FPS_TXT    5
#define SB_FPS        6

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

winTVVClipWidget::winTVVClipWidget(rcfile *_rc, QWidget* parent, const char* name)
  :  KTMainWindow( name ), rc(_rc)
{
#ifdef vcDEBUG
  debug("vc: begin of init");
#endif

  setCaption((const char*)(QString("kWinTV - Video Clip Streamer")));

#if QT_VERSION < 200
  // set icon
  KIconLoader *loader = kapp->getIconLoader();
  pm = loader->loadIcon("kwintv_logo.png");
#else
  pm = DesktopIcon("kwintv_logo.png");
#endif

  setIconText("kWinTV - Video Clip Streamer");
  setIcon(pm);

  /////////////////////////////////////////////////////////////////////////////
  QPopupMenu *filemenu=new QPopupMenu;
  filemenu->insertItem(i18n("&Directory"),this,SLOT(saveAsSLOT()));
  filemenu->insertSeparator();
  filemenu->insertItem(i18n("&Close"),this,SLOT(toggleVClipSLOT()));

//    QPopupMenu *optionsmenu=new QPopupMenu;
//    optionsmenu->insertItem(i18n("&Options"),this,SLOT(optionsSLOT()));

  // menu bar  
  KMenuBar *menubar = new KMenuBar( this, "kwintv -- MenuBar --" );
  menubar->insertItem(i18n("&File"),filemenu);
  menubar->insertItem(i18n("&Options"),this,SLOT(optionsSLOT()));
  setMenu(menubar);

  

  /////////////////////////////////////////////////////////////////////////////
  streamview = new wintvstreamview(rc,this);
  setView(streamview);

  connect(streamview,SIGNAL(closeRequestSIGNAL()),this,SLOT(closeSLOT()));
  
  /////////////////////////////////////////////////////////////////////////////
  // set up ctab dialogs
  tab= new vtab();
  sframeratetab = new sframerate(tab);
  sframeratetab->setData(streamview->getMovieParamsPtr());
  tab->addTab(sframeratetab,"Frames");
  //stimertab = new soptions(tab);
  //tab->addTab(stimertab,"Timer");
  soptionstab = new soptions(tab);
  tab->addTab(soptionstab,"Options");

  tab->setCancelButton();
  tab->setOkButton();
  tab->setDefaultButton();

  // connect ctab dialog apply, cancel, default
  connect(tab,SIGNAL(applyButtonPressed()),this,SLOT(applyTabSLOT())); 
  connect(tab,SIGNAL(cancelButtonPressed()),this,SLOT(cancelTabSLOT())); 
  connect(tab,SIGNAL(defaultButtonPressed()),this,SLOT(defaultTabSLOT()));

  /////////////////////////////////////////////////////////////////////////////
  // set size
  clip=TRUE;

  resize( streamview->width(), streamview->height()+menubar->height() );
  setMinimumSize( streamview->width(), streamview->height()+menubar->height() );
  setMaximumSize( streamview->width(), streamview->height()+menubar->height() );
}

/* ------------------------------------------------------------------------------*/

winTVVClipWidget::~winTVVClipWidget()
{
#ifdef vcDEBUG
  debug("vc: destroying winTVVClipWidget");
#endif

  delete soptionstab;
  delete sframeratetab;
  delete stimertab;
  delete tab;

#ifdef vcDEBUG
  debug("vc: winTVVClipWidget destroyed");
#endif
}

/* ------------------------------------------------------------------------------*/

void winTVVClipWidget::toggleVClipSLOT()
{
  if ( !isVisible() ) 
    show();
  else
    hide();
}

/* ------------------------------------------------------------------------------*/

void winTVVClipWidget::closeSLOT()
{
  // halt video stream

  // hide window
  toggleVClipSLOT();
}

/* ------------------------------------------------------------------------------*/

void winTVVClipWidget::optionsSLOT()
{
  tab->show();
  sframeratetab->setData(streamview->getMovieParamsPtr());
}

/* ------------------------------------------------------------------------------*/

void winTVVClipWidget::saveAsSLOT()
{
  /*
    QString fileName = 
    QFileDialog::getSaveFileName(".",QString("*")+QString(par.extension),this,NULL);
    if ( !fileName.isNull() ) {                 // got a file name
    // copy tmp avi file to file filename
    if (system((const char *)QString("mv ")+QString(par.filename)+QString(" ")+fileName))
    QMessageBox::critical( this, "kWinTV",i18n("Was unable to move file %s to file %s"),
    par.filename,(const char *)fileName);  
    }
  */
}

// =============================================================================== 

void winTVVClipWidget::applyTabSLOT()
{
  tab->hide();
  sframeratetab->getData(streamview->getMovieParamsPtr());
} 
void winTVVClipWidget::defaultTabSLOT()
{
  sframeratetab->setData(streamview->getMovieParamsPtr());
}
void winTVVClipWidget::cancelTabSLOT()
{
  tab->hide();
}

// =============================================================================== 
// =============================================================================== 
// =============================================================================== 
// never change the next line!
#include "wintvstream.moc"

// end of file










